/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache.eviction;

import org.jboss.cache.config.ConfigurationException;
import org.jboss.cache.config.Dynamic;
import org.jboss.cache.config.EvictionAlgorithmConfig;

import java.util.concurrent.TimeUnit;

/**
 * Configuration implementation for {@link LRUPolicy}.
 * <p/>
 * If configured via XML, expects the following:
 * <p/>
 * <pre>
 * <region name="/maxAgeTest/">
 *    <attribute name="maxNodes">10000</attribute>
 *    <attribute name="timeToLiveSeconds">8</attribute>
 *    <attribute name="maxAgeSeconds">10</attribute>
 * </region>
 * </pre>
 *
 * @author Daniel Huang (dhuang@jboss.org)
 * @version $Revision: 6610 $
 * @deprecated see {@link org.jboss.cache.eviction.LRUAlgorithmConfig}
 */
@Deprecated
public class LRUConfiguration extends EvictionPolicyConfigBase implements ModernizableConfig
{
   /**
    * The serialVersionUID
    */
   private static final long serialVersionUID = -3426716488271559729L;

   @Dynamic
   private int timeToLiveSeconds;
   @Dynamic
   private int maxAgeSeconds;

   public LRUConfiguration()
   {
      super();
      // Force config of ttls
      setTimeToLiveSeconds(-1);
   }

   public EvictionAlgorithmConfig modernizeConfig()
   {
      LRUAlgorithmConfig modernCfg = new LRUAlgorithmConfig();
      modernCfg.setMaxNodes(getMaxNodes());
      modernCfg.setMinTimeToLive(getMinTimeToLiveSeconds(), TimeUnit.SECONDS);
      modernCfg.setTimeToLive(getTimeToLiveSeconds(), TimeUnit.SECONDS);
      modernCfg.setMaxAge(getMaxAgeSeconds(), TimeUnit.SECONDS);
      return modernCfg;
   }

   @Override
   protected void setEvictionPolicyClassName()
   {
      setEvictionPolicyClass(LRUPolicy.class.getName());
   }

   public int getTimeToLiveSeconds()
   {
      return timeToLiveSeconds;
   }

   public void setTimeToLiveSeconds(int timeToLiveSeconds)
   {
      testImmutability("timeToLiveSeconds");
      this.timeToLiveSeconds = timeToLiveSeconds;
   }

   public int getMaxAgeSeconds()
   {
      return maxAgeSeconds;
   }

   public void setMaxAgeSeconds(int maxAgeSeconds)
   {
      testImmutability("maxAgeSeconds");
      this.maxAgeSeconds = maxAgeSeconds;
   }

   /**
    * Requires a positive timeToLive value or ConfigurationException
    * is thrown.
    */
   @Override
   public void validate() throws ConfigurationException
   {
      if (timeToLiveSeconds < 0)
      {
         throw new ConfigurationException("timeToLive must be " +
               "configured to a value greater than or equal to 0 for " + getEvictionPolicyClass());
      }
   }


   public String toString()
   {
      return "LRUConfiguration{" +
            "timeToLiveSeconds=" + timeToLiveSeconds +
            ", timeToLiveSeconds=" + timeToLiveSeconds +
            ", maxAgeSeconds=" + maxAgeSeconds +
            '}';
   }

   public boolean equals(Object o)
   {
      if (this == o) return true;
      if (!(o instanceof LRUConfiguration)) return false;
      if (!super.equals(o)) return false;

      LRUConfiguration that = (LRUConfiguration) o;

      if (maxAgeSeconds != that.maxAgeSeconds) return false;
      if (maxAgeSeconds != that.maxAgeSeconds) return false;
      if (timeToLiveSeconds != that.timeToLiveSeconds) return false;
      if (timeToLiveSeconds != that.timeToLiveSeconds) return false;

      return true;
   }

   public int hashCode()
   {
      int result = super.hashCode();
      result = 31 * result + timeToLiveSeconds;
      result = 31 * result + maxAgeSeconds;
      result = 31 * result + (timeToLiveSeconds ^ (timeToLiveSeconds >>> 7));
      result = 31 * result + (maxAgeSeconds ^ (maxAgeSeconds >>> 7));
      return result;
   }

   @Override
   public void reset()
   {
      super.reset();
      setTimeToLiveSeconds(-1);
   }

   @Override
   public LRUConfiguration clone() throws CloneNotSupportedException
   {
      return (LRUConfiguration) super.clone();
   }

}
