package org.jboss.cache.factories;

import org.jboss.cache.factories.annotations.DefaultFactoryFor;
import org.jboss.cache.lock.LockManager;
import org.jboss.cache.lock.MVCCLockManager;
import org.jboss.cache.lock.NodeBasedLockManager;
import org.jboss.cache.lock.PessimisticNodeBasedLockManager;

/**
 * Creates lock managers
 *
 * @author Manik Surtani (<a href="mailto:manik@jboss.org">manik@jboss.org</a>)
 * @since 2.2.0
 */
@DefaultFactoryFor(classes = LockManager.class)
public class LockManagerFactory extends EmptyConstructorFactory
{
   @Override
   @SuppressWarnings("deprecation")
   protected <T> T construct(Class<T> componentType)
   {
      if (log.isTraceEnabled()) log.trace("Cache configuration is " + configuration.getNodeLockingScheme());
      switch (configuration.getNodeLockingScheme())
      {
         case MVCC:
            return componentType.cast(super.construct(MVCCLockManager.class));
         case OPTIMISTIC:
            return componentType.cast(super.construct(NodeBasedLockManager.class));
         case PESSIMISTIC:
         default:
            return componentType.cast(super.construct(PessimisticNodeBasedLockManager.class));
      }
   }
}
