/*
 * JBoss, the OpenSource J2EE webOS
 * 
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache.factories;

import org.jboss.cache.factories.annotations.DefaultFactoryFor;
import org.jboss.cache.statetransfer.DefaultStateTransferGenerator;
import org.jboss.cache.statetransfer.DefaultStateTransferIntegrator;
import org.jboss.cache.statetransfer.LegacyStateTransferGenerator;
import org.jboss.cache.statetransfer.LegacyStateTransferIntegrator;
import org.jboss.cache.statetransfer.StateTransferGenerator;
import org.jboss.cache.statetransfer.StateTransferIntegrator;

/**
 * Factory class able to create {@link org.jboss.cache.statetransfer.StateTransferGenerator} and
 * {@link org.jboss.cache.statetransfer.StateTransferIntegrator} instances.
 * <p/>
 * Updated in 3.0.0 to extend ComponentFactory, etc.
 * <p/>
 *
 * @author <a href="brian.stansberry@jboss.com">Brian Stansberry</a>
 * @author Manik Surtani
 * @version $Revision: 6530 $
 */
@DefaultFactoryFor(classes = {StateTransferGenerator.class, StateTransferIntegrator.class})
public class StateTransferFactory extends ComponentFactory
{
   @SuppressWarnings("deprecation")
   protected <T> T construct(Class<T> componentType)
   {
      if (componentType.equals(StateTransferIntegrator.class))
      {
         switch (configuration.getNodeLockingScheme())
         {
            case MVCC:
               return componentType.cast(new DefaultStateTransferIntegrator());
            default:
               return componentType.cast(new LegacyStateTransferIntegrator());
         }
      }
      else
      {
         switch (configuration.getNodeLockingScheme())
         {
            case MVCC:
               return componentType.cast(new DefaultStateTransferGenerator());
            default:
               return componentType.cast(new LegacyStateTransferGenerator());
         }
      }
   }
}
